#if !defined HATP_LIGHT_WORLDBASEAGENT_HH_
#define HATP_LIGHT_WORLDBASEAGENT_HH_

#include<string>
#include<vector>
#include<map>

#include "LightEntity.hh"
#include "WorldBaseAgent.hh"

/** Represent what an agent knows about the world (and the entities).
 * This is related to the belief management integrated in HATP.
 */
class LightWorldBaseAgent {

private:
	/** Is used when returning a reference to an entity after an error in operator[](const char* c). */
	LightEntity defaultLightEntity;
	/** Contains the set of entities that the agent knows. */
	std::map<std::string, LightEntity*> database;

public:
	explicit LightWorldBaseAgent();
	explicit LightWorldBaseAgent(WorldBaseAgent const& wb, std::map<std::string, LightEntity*> ldb);
	LightWorldBaseAgent(LightWorldBaseAgent const& lwb, std::map<LightEntity*, LightEntity*> ldb);
	std::map<std::string, LightEntity*> const& getDatabase() const;
	void replaceEntity(std::string const& sName, LightEntity* e);
	bool containEntity(std::string const& sName) const;
	bool containEntity(std::string const& sName, std::string const& sType) const;
	LightEntity* find(std::string const& sName);
	LightEntity const* find(std::string const& sName) const;
	LightEntity* find(std::string const& sName, std::string const& sType);
	LightEntity const* find(std::string const& sName, std::string const& sType) const;
	std::vector<LightEntity*> findByType(std::string const& sType) const;
	std::string toString() const;
	LightEntity& operator[](const char* c);
	LightEntity& operator[](std::string const& s);
	LightEntity const& operator[](const char* c) const;
	LightEntity const& operator[](std::string const& s) const;
	LightWorldBaseAgent& operator=(LightWorldBaseAgent const& lwb);
	~LightWorldBaseAgent();
};

/** Default constructor.
 * All constructors set defaultEntity to an empty LightEntity.
 */
inline LightWorldBaseAgent::LightWorldBaseAgent() {}

/** Construct from a WorldBaseAgent and a given LightEntity database
 * All constructors set defaultEntity to an empty LightEntity.
 * @param wb WorldBaseAgent database to copy
 * @param ldb LightEntity database
 */
inline LightWorldBaseAgent::LightWorldBaseAgent(WorldBaseAgent const& wb, std::map<std::string, LightEntity*> ldb) {
	LightEntity *e;
	std::map<std::string, Entity*> tmp=wb.getDatabase();
	std::map<std::string, Entity*>::const_iterator it;
	for(it = tmp.begin() ; it!=tmp.end() ; ++it) {
		e = new LightEntity(*(it->second));
		database[it->first] = e;
	}
	// we need to update the LightEntity* attributes
	std::map<std::string, Entity*> atomsMap;
	std::map<std::string, Entity*>::const_iterator itAtoms;
	std::map<std::string, std::set<Entity*> > setsMap;
	std::map<std::string, std::set<Entity*> >::const_iterator itSets;
	std::set<Entity*>::const_iterator itS;
	for(it = tmp.begin() ; it!=tmp.end() ; ++it) { // for each entity in wba
		atomsMap = it->second->getEntityAttrs().getStaticAtomAttrs();
		for(itAtoms = atomsMap.begin() ; itAtoms != atomsMap.end() ; ++itAtoms) {
			if(itAtoms->second != NULL) {
					database[it->first]->get<LightEntity*>(itAtoms->first) = ldb[itAtoms->second->getName()];
				}
		}
		atomsMap = it->second->getEntityAttrs().getDynamicAtomAttrs();
		for(itAtoms = atomsMap.begin() ; itAtoms != atomsMap.end() ; ++itAtoms) { // for each dynamic atom attribute
			if(itAtoms->second != NULL) {
				database[it->first]->get<LightEntity*>(itAtoms->first) = ldb[itAtoms->second->getName()];
			}
		}
		setsMap = it->second->getEntityAttrs().getStaticSetAttrs(); 
		for(itSets = setsMap.begin() ; itSets != setsMap.end() ; itSets++) { // for each set
			database[it->first]->get< std::set<LightEntity*> >(itSets->first).clear();
			for(itS = (itSets->second).begin() ; itS != (itSets->second).end() ; ++itS) { // for each elt of this set
				database[it->first]->get< std::set<LightEntity*> >(itSets->first).insert(ldb[(*itS)->getName()]);
			}
		}
		setsMap = it->second->getEntityAttrs().getDynamicSetAttrs();
		for(itSets = setsMap.begin() ; itSets != setsMap.end() ; itSets++) {
			database[it->first]->get< std::set<LightEntity*> >(itSets->first).clear();
			for(itS = (itSets->second).begin() ; itS != (itSets->second).end() ; ++itS) {
				database[it->first]->get< std::set<LightEntity*> >(itSets->first).insert(ldb[(*itS)->getName()]);
			}
		}
	}
}

/** Construct from a LightWorldBaseAgent and a given LightEntity database
 * All constructors set defaultEntity to an empty LightEntity.
 * @param lwb LightWorldBaseAgent database to copy
 * @param ldb LightEntity database
 */
inline LightWorldBaseAgent::LightWorldBaseAgent(LightWorldBaseAgent const& lwb, std::map<LightEntity*, LightEntity*> ldb) {
	LightEntity *e;
	std::map<std::string, LightEntity*> const& tmp=lwb.getDatabase();
	for(std::map<std::string, LightEntity*>::const_iterator it = tmp.begin() ; it!=tmp.end() ; ++it) {
		e = new LightEntity(*(it->second));
		database[it->first] = e;
	}
	// we need to update the Entity* attributes
	for(std::map<std::string, LightEntity*>::iterator itD = database.begin() ; itD!=database.end() ; ++itD)
		itD->second->updateAttrs(ldb);
}

/** Gives a reference to the entity database for this agent.
 * @return LightEntity database
 */
inline std::map<std::string, LightEntity*> const& LightWorldBaseAgent::getDatabase() const {
	return database;
}

/** Find the given LightEntity by the provided name.
 * @param sName LightEntity to find
 * @return A pointer to the LightEntity or NULL if not found
 */
inline LightEntity* LightWorldBaseAgent::find(std::string const& sName) {
	std::map<std::string, LightEntity*>::iterator it = database.find(sName);
	if(it != database.end()) return it->second;
	return NULL;
}

/** Find the given LightEntity by the provided name.
 * @param sName LightEntity to find
 * @return A const pointer to the LightEntity or NULL if not found
 */
inline LightEntity const* LightWorldBaseAgent::find(std::string const& sName) const {
	std::map<std::string, LightEntity*>::const_iterator it = database.find(sName);
	if(it != database.end()) return it->second;
	return NULL;
}

/** Assignement operator.
 * @param lwb LightWorldBaseAgent to copy
 * @return Reference to this
 */
inline LightWorldBaseAgent& LightWorldBaseAgent::operator=(LightWorldBaseAgent const& lwb) {
	// we empty database
	for(std::map<std::string, LightEntity*>::iterator itD = database.begin() ; itD != database.end() ; ++itD)
		delete itD->second;
	database.clear();
	std::map<LightEntity*, LightEntity*> oldToNew; // to keep correspondence
	LightEntity *e;
	std::map<std::string, LightEntity*> const& tmp=lwb.getDatabase();
	for(std::map<std::string, LightEntity*>::const_iterator it = tmp.begin() ; it!=tmp.end() ; ++it) {
		e = new LightEntity(*(it->second));
		database[it->first] = e;
		oldToNew[it->second] = e;
	}
	// we need to update the Entity* attributes
	for(std::map<std::string, LightEntity*>::iterator itD = database.begin() ; itD!=database.end() ; ++itD)
		itD->second->updateAttrs(oldToNew);
	return *this;
}

#endif
