/*
 * Copyright (C) 2007 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#if !defined HATP_PLAN_STOCK_MANAGER_HH_
#define HATP_PLAN_STOCK_MANAGER_HH_

#include<map>
#include<vector>
#include<boost/date_time/posix_time/posix_time.hpp>

#include "Plan.hh"
#include "PlanTree.hh"
#include "TaskList.hh"

class PlanStockManager
{
private:
	std::multimap<float, Plan*> plans;
	boost::posix_time::ptime start_time;
	unsigned long int NumberOfPlans;
	bool solutionExists;
	std::string lastMessage;

public:
	explicit PlanStockManager();
	void setSolutionExists(bool b);
	bool getSolutionExists() const;
	float getBestScore() const;
	int getNumberOfPlans() const;
	std::map<std::string, float> const* getBestPlanDetails() const;
	void addPlan(Plan* p);
	void setStartTime(boost::posix_time::ptime const& argTime);
	std::vector<Plan*> getPlans() const;
	Plan* getBestPlan() const;
	Plan* getNextPlan();
	Plan* getPlanByID(unsigned long int id) const;
	std::vector<Plan*> getBestPlans() const;
	unsigned int getNbPlans() const;
	void clear();
	std::string getLastMessage() const;
	~PlanStockManager();
};

/**
 * constructor: sets plans to an empty map
 */
inline PlanStockManager::PlanStockManager() {
	NumberOfPlans = 0;
	solutionExists = false;
	lastMessage = "";
}

inline void PlanStockManager::setSolutionExists(bool b) {
	solutionExists = b;
}

inline bool PlanStockManager::getSolutionExists() const {
	return solutionExists;
}

inline std::string PlanStockManager::getLastMessage() const {
	return lastMessage;
}

/**
 * return -1 if plans is empty else return the score of the first member in plans
 */
inline float PlanStockManager::getBestScore() const {
	if(plans.empty()) return -1;
	else return (plans.begin()->first);
}


/**
 * return the number of plans found
 */
inline int PlanStockManager::getNumberOfPlans() const {
	return NumberOfPlans;
}

/**
 * return NULL if plans is empty else return a pointer on score details of the first member in plans
 */
inline std::map<std::string, float> const* PlanStockManager::getBestPlanDetails() const {
	if(plans.empty()) return (std::map<std::string, float> const*)NULL;
	else return plans.begin()->second->getPtrScoreDetails();
}

/**
 * set start_time to argTime
 */
inline void PlanStockManager::setStartTime(boost::posix_time::ptime const& argTime) {
	start_time = argTime;
}

/**
 * returns plans size
 */
inline unsigned int PlanStockManager::getNbPlans() const {
	return plans.size();
}

#endif
